@@ -26,6 +26,9 @@
 #include "access/xact.h"
 #include "access/xlog.h"
 #include "catalog/pg_extension.h"
+#if (PG_VERSION_NUM >= 160000)
+#include "catalog/pg_database.h"
+#endif
 #include "catalog/indexing.h"
 #include "catalog/namespace.h"
 #include "commands/extension.h"
@@ -37,6 +40,7 @@
 #include "utils/acl.h"
 #include "utils/builtins.h"
 #include "utils/fmgroids.h"
+#include "utils/formatting.h"
 #include "utils/inval.h"
 #include "utils/lsyscache.h"
 #include "utils/memutils.h"
@@ -88,6 +92,9 @@ static void AlterJob(int64 jobId, text *scheduleText, text *commandText,
 						text *databaseText, text *usernameText, bool *active);
 
 static Oid GetRoleOidIfCanLogin(char *username);
+static entry * ParseSchedule(char *scheduleText);
+static bool TryParseInterval(char *scheduleText, uint32 *secondsInterval);
+
 
 /* SQL-callable functions */
 PG_FUNCTION_INFO_V1(cron_schedule);
@@ -215,12 +222,14 @@ ScheduleCronJob(text *scheduleText, text *commandText, text *databaseText,
 
 	/* check schedule is valid */
 	schedule = text_to_cstring(scheduleText);
-	parsedSchedule = parse_cron_entry(schedule);
+	parsedSchedule = ParseSchedule(schedule);
 
 	if (parsedSchedule == NULL)
 	{
 		ereport(ERROR, (errcode(ERRCODE_INVALID_PARAMETER_VALUE),
-						errmsg("invalid schedule: %s", schedule)));
+						errmsg("invalid schedule: %s", schedule),
+						errhint("Use cron format (e.g. 5 4 * * *), or interval "
+								"format '[1-59] seconds'")));
 	}
 
 	free_entry(parsedSchedule);
@@ -297,8 +306,14 @@ ScheduleCronJob(text *scheduleText, text *commandText, text *databaseText,
 	}
 
 	/* ensure the user that is used in the job can connect to the database */
+#if (PG_VERSION_NUM >= 160000)
+	aclresult = object_aclcheck(DatabaseRelationId,
+								get_database_oid(database_name, false),
+								userIdcheckacl, ACL_CONNECT);
+#else
 	aclresult = pg_database_aclcheck(get_database_oid(database_name, false),
 										userIdcheckacl, ACL_CONNECT);
+#endif
 
 	if (aclresult != ACLCHECK_OK)
 		elog(ERROR, "User %s does not have CONNECT privilege on %s",
@@ -522,8 +537,8 @@ NextRunId(void)
 	Datum sequenceIdDatum = InvalidOid;
 	Oid savedUserId = InvalidOid;
 	int savedSecurityContext = 0;
-	Datum jobIdDatum = 0;
-	int64 jobId = 0;
+	Datum runIdDatum = 0;
+	int64 runId = 0;
 	bool failOK = true;
 	MemoryContext originalContext = CurrentMemoryContext;
 
@@ -551,17 +566,17 @@ NextRunId(void)
 	SetUserIdAndSecContext(CronExtensionOwner(), SECURITY_LOCAL_USERID_CHANGE);
 
 	/* generate new and unique colocation id from sequence */
-	jobIdDatum = DirectFunctionCall1(nextval_oid, sequenceIdDatum);
+	runIdDatum = DirectFunctionCall1(nextval_oid, sequenceIdDatum);
 
 	SetUserIdAndSecContext(savedUserId, savedSecurityContext);
 
-	jobId = DatumGetInt64(jobIdDatum);
+	runId = DatumGetInt64(runIdDatum);
 
 	PopActiveSnapshot();
 	CommitTransactionCommand();
 	MemoryContextSwitchTo(originalContext);
 
-	return jobId;
+	return runId;
 }
 
 /*
@@ -662,8 +677,8 @@ cron_unschedule(PG_FUNCTION_ARGS)
 Datum
 cron_unschedule_named(PG_FUNCTION_ARGS)
 {
-	Datum jobNameDatum = PG_GETARG_DATUM(0);
-	Name jobName = DatumGetName(jobNameDatum);
+	Datum jobNameDatum = 0;
+	char *jobName = NULL;
 
 	Oid userId = GetUserId();
 	char *userName = GetUserNameFromId(userId, false);
@@ -676,10 +691,18 @@ cron_unschedule_named(PG_FUNCTION_ARGS)
 	bool indexOK = false;
 	HeapTuple heapTuple = NULL;
 
+	if (PG_ARGISNULL(0))
+	{
+		ereport(ERROR, (errmsg("job_name can not be NULL")));
+	}
+
+	jobNameDatum = PG_GETARG_DATUM(0);
+	jobName = TextDatumGetCString(jobNameDatum);
+
 	cronJobsTable = table_open(CronJobRelationId(), RowExclusiveLock);
 
 	ScanKeyInit(&scanKey[0], Anum_cron_job_jobname,
-				BTEqualStrategyNumber, F_NAMEEQ, jobNameDatum);
+				BTEqualStrategyNumber, F_TEXTEQ, jobNameDatum);
 	ScanKeyInit(&scanKey[1], Anum_cron_job_username,
 				BTEqualStrategyNumber, F_TEXTEQ, userNameDatum);
 
@@ -690,7 +713,7 @@ cron_unschedule_named(PG_FUNCTION_ARGS)
 	if (!HeapTupleIsValid(heapTuple))
 	{
 		ereport(ERROR, (errmsg("could not find valid entry for job '%s'",
-							   NameStr(*jobName))));
+							   jobName)));
 	}
 
 	EnsureDeletePermission(cronJobsTable, heapTuple);
@@ -844,7 +867,6 @@ LoadCronJobList(void)
 		PopActiveSnapshot();
 		CommitTransactionCommand();
 		MemoryContextSwitchTo(originalContext);
-		pgstat_report_activity(STATE_IDLE, NULL);
 
 		return NIL;
 	}
@@ -897,7 +919,6 @@ LoadCronJobList(void)
 	PopActiveSnapshot();
 	CommitTransactionCommand();
 	MemoryContextSwitchTo(originalContext);
-	pgstat_report_activity(STATE_IDLE, NULL);
 
 	return jobList;
 }
@@ -961,7 +982,15 @@ TupleToCronJob(TupleDesc tupleDescriptor, HeapTuple heapTuple)
 									 tupleDescriptor, &isJobNameNull);
 		if (!isJobNameNull)
 		{
-			job->jobName = DatumGetName(jobName);
+			/* Handle the column type change introduced in 1.5 */
+			if (TupleDescAttr(tupleDescriptor, Anum_cron_job_jobname - 1)->atttypid == NAMEOID)
+			{
+				job->jobName = pstrdup(NameStr(*DatumGetName(jobName)));
+			}
+			else
+			{
+				job->jobName = TextDatumGetCString(jobName);
+			}
 		}
 		else
 		{
@@ -969,7 +998,7 @@ TupleToCronJob(TupleDesc tupleDescriptor, HeapTuple heapTuple)
 		}
 	}
 
-	parsedSchedule = parse_cron_entry(job->scheduleText);
+	parsedSchedule = ParseSchedule(job->scheduleText);
 	if (parsedSchedule != NULL)
 	{
 		/* copy the schedule and free the allocated memory immediately */
@@ -1082,8 +1111,6 @@ InsertJobRunDetail(int64 runId, int64 *jobId, char *database, char *username, ch
 	argTypes[5] = TEXTOID;
 	argValues[5] = CStringGetTextDatum(status);
 
-	pgstat_report_activity(STATE_RUNNING, querybuf.data);
-
 	if(SPI_execute_with_args(querybuf.data,
 		argCount, argTypes, argValues, NULL, false, 1) != SPI_OK_INSERT)
 		elog(ERROR, "SPI_exec failed: %s", querybuf.data);
@@ -1094,7 +1121,6 @@ InsertJobRunDetail(int64 runId, int64 *jobId, char *database, char *username, ch
 	PopActiveSnapshot();
 	CommitTransactionCommand();
 	MemoryContextSwitchTo(originalContext);
-	pgstat_report_activity(STATE_IDLE, NULL);
 }
 
 void
@@ -1186,8 +1212,6 @@ UpdateJobRunDetail(int64 runId, int32 *job_pid, char *status, char *return_messa
 	/* and add the where clause */
 	appendStringInfo(&querybuf, " where runid = $%d", i);
 
-	pgstat_report_activity(STATE_RUNNING, querybuf.data);
-
 	if(SPI_execute_with_args(querybuf.data,
 		i, argTypes, argValues, NULL, false, 1) != SPI_OK_UPDATE)
 		elog(ERROR, "SPI_exec failed: %s", querybuf.data);
@@ -1198,7 +1222,6 @@ UpdateJobRunDetail(int64 runId, int32 *job_pid, char *status, char *return_messa
 	PopActiveSnapshot();
 	CommitTransactionCommand();
 	MemoryContextSwitchTo(originalContext);
-	pgstat_report_activity(STATE_IDLE, NULL);
 }
 
 
@@ -1267,7 +1290,14 @@ AlterJob(int64 jobId, text *scheduleText, text *commandText, text *databaseText,
 	{
 		database_name = text_to_cstring(databaseText);
 		/* ensure the user that is used in the job can connect to the database */
-		aclresult = pg_database_aclcheck(get_database_oid(database_name, false), userIdcheckacl, ACL_CONNECT);
+#if (PG_VERSION_NUM >= 160000)
+		aclresult = object_aclcheck(DatabaseRelationId,
+									get_database_oid(database_name, false),
+									userIdcheckacl, ACL_CONNECT);
+#else
+		aclresult = pg_database_aclcheck(get_database_oid(database_name, false),
+										 userIdcheckacl, ACL_CONNECT);
+#endif
 
 		if (aclresult != ACLCHECK_OK)
 			elog(ERROR, "User %s does not have CONNECT privilege on %s", GetUserNameFromId(userIdcheckacl, false), database_name);
@@ -1282,12 +1312,14 @@ AlterJob(int64 jobId, text *scheduleText, text *commandText, text *databaseText,
 	if (scheduleText != NULL)
 	{
 		schedule = text_to_cstring(scheduleText);
-		parsedSchedule = parse_cron_entry(schedule);
+		parsedSchedule = ParseSchedule(schedule);
 
 		if (parsedSchedule == NULL)
 		{
 			ereport(ERROR, (errcode(ERRCODE_INVALID_PARAMETER_VALUE),
-					errmsg("invalid schedule: %s", schedule)));
+					errmsg("invalid schedule: %s", schedule),
+					errhint("Use cron format (e.g. 5 4 * * *), or interval "
+							"format '[1-59] seconds'")));
 		}
 
 		free_entry(parsedSchedule);
@@ -1396,8 +1428,6 @@ MarkPendingRunsAsFailed(void)
 		, CRON_SCHEMA_NAME, JOB_RUN_DETAILS_TABLE_NAME, GetCronStatus(CRON_STATUS_FAILED), GetCronStatus(CRON_STATUS_STARTING), GetCronStatus(CRON_STATUS_RUNNING));
 
 
-	pgstat_report_activity(STATE_RUNNING, querybuf.data);
-
 	if (SPI_exec(querybuf.data, 0) != SPI_OK_UPDATE)
 		elog(ERROR, "SPI_exec failed: %s", querybuf.data);
 
@@ -1407,7 +1437,6 @@ MarkPendingRunsAsFailed(void)
 	PopActiveSnapshot();
 	CommitTransactionCommand();
 	MemoryContextSwitchTo(originalContext);
-	pgstat_report_activity(STATE_IDLE, NULL);
 }
 
 char *
@@ -1467,3 +1496,74 @@ JobTableExists(void)
 
 	return jobTableOid != InvalidOid;
 }
+
+
+/*
+ * ParseSchedule attempts to parse a cron schedule or an interval in seconds.
+ * The returned pointer is allocated using malloc and should be freed by the
+ * caller.
+ */
+static entry *
+ParseSchedule(char *scheduleText)
+{
+	uint32 secondsInterval = 0;
+	entry *schedule;
+
+	/*
+	 * First try to parse as a cron schedule.
+	 */
+	schedule = parse_cron_entry(scheduleText);
+	if (schedule != NULL)
+	{
+		/* valid cron schedule */
+		return schedule;
+	}
+
+	/*
+	 * Parse as interval on seconds.
+	 */
+	if (TryParseInterval(scheduleText, &secondsInterval))
+	{
+		schedule = calloc(sizeof(entry), sizeof(char));
+		schedule->secondsInterval = secondsInterval;
+		return schedule;
+	}
+
+	elog(LOG, "failed to parse schedule: %s", scheduleText);
+	return NULL;
+}
+
+
+/*
+ * TryParseInterval returns whether scheduleText is of the form
+ * <positive number> second[s].
+ */
+static bool
+TryParseInterval(char *scheduleText, uint32 *secondsInterval)
+{
+	char lastChar = '\0';
+	char plural = '\0';
+	char extra = '\0';
+	char *lowercaseSchedule = asc_tolower(scheduleText, strlen(scheduleText));
+
+	int numParts = sscanf(lowercaseSchedule, " %u secon%c%c %c", secondsInterval,
+						  &lastChar, &plural, &extra);
+	if (lastChar != 'd')
+	{
+		/* value did not have a "second" suffix */
+		return false;
+	}
+
+	if (numParts == 2)
+	{
+		/* <number> second (allow "2 second") */
+		return 0 < *secondsInterval && *secondsInterval < 60;
+	}
+	else if (numParts == 3 && plural == 's')
+	{
+		/* <number> seconds (allow "1 seconds") */
+		return 0 < *secondsInterval && *secondsInterval < 60;
+	}
+
+	return false;
+}
