@@ -60,6 +60,11 @@
 #include "commands/extension.h"
 #include "commands/sequence.h"
 #include "commands/trigger.h"
+#if (PG_VERSION_NUM >= 160000)
+#include "utils/guc_hooks.h"
+#else
+#include "commands/variable.h"
+#endif
 #include "lib/stringinfo.h"
 #include "libpq-fe.h"
 #include "libpq/pqmq.h"
@@ -117,9 +122,8 @@ void _PG_init(void);
 void _PG_fini(void);
 static void pg_cron_sigterm(SIGNAL_ARGS);
 static void pg_cron_sighup(SIGNAL_ARGS);
-static void pg_cron_background_worker_sigterm(SIGNAL_ARGS);
-void PgCronLauncherMain(Datum arg);
-void CronBackgroundWorker(Datum arg);
+PGDLLEXPORT void PgCronLauncherMain(Datum arg);
+PGDLLEXPORT void CronBackgroundWorker(Datum arg);
 
 static void StartAllPendingRuns(List *taskList, TimestampTz currentTime);
 static void StartPendingRuns(CronTask *task, ClockProgress clockProgress,
@@ -138,7 +142,7 @@ static void ManageCronTasks(List *taskList, TimestampTz currentTime);
 static void ManageCronTask(CronTask *task, TimestampTz currentTime);
 static void ExecuteSqlString(const char *sql);
 static void GetTaskFeedback(PGresult *result, CronTask *task);
-static void GetBgwTaskFeedback(shm_mq_handle *responseq, CronTask *task, bool running);
+static void ProcessBgwTaskFeedback(CronTask *task, bool running);
 
 static bool jobCanceled(CronTask *task);
 static bool jobStartupTimeout(CronTask *task, TimestampTz currentTime);
@@ -163,6 +167,8 @@ static int MaxRunningTasks = 0;
 static int CronLogMinMessages = WARNING;
 static bool UseBackgroundWorkers = false;
 
+char  *cron_timezone = NULL;
+
 static const struct config_enum_entry cron_message_level_options[] = {
 	{"debug5", DEBUG5, false},
 	{"debug4", DEBUG4, false},
@@ -262,6 +268,16 @@ _PG_init(void)
 		GUC_SUPERUSER_ONLY,
 		NULL, NULL, NULL);
 
+	DefineCustomBoolVariable(
+		"cron.launch_active_jobs",
+		gettext_noop("Launch jobs that are defined as active."),
+		NULL,
+		&LaunchActiveJobs,
+		true,
+		PGC_SIGHUP,
+		GUC_SUPERUSER_ONLY,
+		NULL, NULL, NULL);
+
 	if (!UseBackgroundWorkers)
 		DefineCustomIntVariable(
 			"cron.max_running_jobs",
@@ -298,6 +314,16 @@ _PG_init(void)
 		GUC_SUPERUSER_ONLY,
 		NULL, NULL, NULL);
 
+	DefineCustomStringVariable(
+		"cron.timezone",
+		gettext_noop("Specify timezone used for cron schedule."),
+		NULL,
+		&cron_timezone,
+		"GMT",
+		PGC_POSTMASTER,
+		GUC_SUPERUSER_ONLY,
+		check_timezone, NULL, NULL);
+
 	/* set up common data for all our workers */
 	worker.bgw_flags = BGWORKER_SHMEM_ACCESS | BGWORKER_BACKEND_DATABASE_CONNECTION;
 	worker.bgw_start_time = BgWorkerStart_RecoveryFinished;
@@ -521,29 +547,6 @@ bgw_generate_returned_message(StringInfoData *display_msg, ErrorData edata)
 		appendStringInfo(display_msg, "\nCONTEXT: %s", edata.context);
 }
 
-/*
- * Signal handler for SIGTERM for background workers
- * 		When we receive a SIGTERM, we set InterruptPending and ProcDiePending
- * 		just like a normal backend.  The next CHECK_FOR_INTERRUPTS() will do the
- * 		right thing.
- */
-static void
-pg_cron_background_worker_sigterm(SIGNAL_ARGS)
-{
-	int save_errno = errno;
-
-	if (MyProc)
-		SetLatch(&MyProc->procLatch);
-
-	if (!proc_exit_inprogress)
-	{
-		InterruptPending = true;
-		ProcDiePending = true;
-	}
-
-	errno = save_errno;
-}
-
 
 /*
  * PgCronLauncherMain is the main entry-point for the background worker
@@ -630,11 +633,6 @@ PgCronLauncherMain(Datum arg)
 
 		AcceptInvalidationMessages();
 
-		if (!CronJobCacheValid)
-		{
-			RefreshTaskHash();
-		}
-
 		if (CronReloadConfig)
 		{
 			/* set the desired log_min_messages */
@@ -644,6 +642,16 @@ PgCronLauncherMain(Datum arg)
 			CronReloadConfig = false;
 		}
 
+		/*
+		 * Both CronReloadConfig and CronJobCacheValid are triggered by SIGHUP.
+		 * ProcessConfigFile should come first, because RefreshTaskHash depends
+		 * on settings that might have changed.
+		 */
+		if (!CronJobCacheValid)
+		{
+			RefreshTaskHash();
+		}
+
 		taskList = CurrentTaskList();
 		currentTime = GetCurrentTimestamp();
 
@@ -657,7 +665,8 @@ PgCronLauncherMain(Datum arg)
 
 	ereport(LOG, (errmsg("pg_cron scheduler shutting down")));
 
-	proc_exit(0);
+	/* return error code to trigger restart */
+	proc_exit(1);
 }
 
 
@@ -684,7 +693,8 @@ StartAllPendingRuns(List *taskList, TimestampTz currentTime)
 			CronJob *cronJob = GetCronJob(task->jobId);
 			entry *schedule = &cronJob->schedule;
 
-			if (schedule->flags & WHEN_REBOOT)
+			if (schedule->flags & WHEN_REBOOT &&
+				task->isActive)
 			{
 				task->pendingRunCount += 1;
 			}
@@ -693,6 +703,27 @@ StartAllPendingRuns(List *taskList, TimestampTz currentTime)
 		RebootJobsScheduled = true;
 	}
 
+	foreach(taskCell, taskList)
+	{
+		CronTask *task = (CronTask *) lfirst(taskCell);
+
+		if (task->secondsInterval > 0 && task->isActive)
+		{
+			/*
+			 * For interval jobs, if a task takes longer than the interval,
+			 * we only queue up once. So if a task that is supposed to run
+			 * every 30 seconds takes 5 minutes, we start another run
+			 * immediately after 5 minutes, but then return to regular cadence.
+			 */
+			if (task->pendingRunCount == 0 &&
+				TimestampDifferenceExceeds(task->lastStartTime, currentTime,
+										   task->secondsInterval * 1000))
+			{
+				task->pendingRunCount += 1;
+			}
+		}
+	}
+
 	if (lastMinute == 0)
 	{
 		lastMinute = TimestampMinuteStart(currentTime);
@@ -933,21 +964,30 @@ static bool
 ShouldRunTask(entry *schedule, TimestampTz currentTime, bool doWild,
 			  bool doNonWild)
 {
-	time_t currentTime_t = timestamptz_to_time_t(currentTime);
-	struct tm *tm = gmtime(&currentTime_t);
+	pg_time_t currentTime_t = timestamptz_to_time_t(currentTime);
+	pg_time_t tomorrowTime_t = timestamptz_to_time_t(currentTime + USECS_PER_DAY);
+	struct pg_tm* cur_tm = pg_localtime(&currentTime_t, pg_tzset(cron_timezone));
 
-	int minute = tm->tm_min -FIRST_MINUTE;
-	int hour = tm->tm_hour -FIRST_HOUR;
-	int dayOfMonth = tm->tm_mday -FIRST_DOM;
-	int month = tm->tm_mon +1 -FIRST_MONTH;
-	int dayOfWeek = tm->tm_wday -FIRST_DOW;
+	int minute = cur_tm->tm_min -FIRST_MINUTE;
+	int hour = cur_tm->tm_hour -FIRST_HOUR;
+	int dayOfMonth = cur_tm->tm_mday -FIRST_DOM;
+	int month = cur_tm->tm_mon +1 -FIRST_MONTH;
+	int dayOfWeek = cur_tm->tm_wday -FIRST_DOW;
+
+	/*
+	 * pg_localtime returns a pointer to a global struct,
+	 * so cur_tm cannot be used after this point.
+	 */
+	struct pg_tm* tomorrow_tm = pg_localtime(&tomorrowTime_t, pg_tzset(cron_timezone));
+	bool lastdom = (schedule->flags & DOM_LAST) != 0 && tomorrow_tm->tm_mday == 1;
+	bool thisdom = lastdom || bit_test(schedule->dom, dayOfMonth) != 0;
+	bool thisdow = bit_test(schedule->dow, dayOfWeek);
 
 	if (bit_test(schedule->minute, minute) &&
 	    bit_test(schedule->hour, hour) &&
 	    bit_test(schedule->month, month) &&
-	    ( ((schedule->flags & DOM_STAR) || (schedule->flags & DOW_STAR))
-	      ? (bit_test(schedule->dow,dayOfWeek) && bit_test(schedule->dom,dayOfMonth))
-	      : (bit_test(schedule->dow,dayOfWeek) || bit_test(schedule->dom,dayOfMonth)))) {
+	    ( (schedule->flags & (DOM_STAR|DOW_STAR)) != 0
+	      ? (thisdom && thisdow) : (thisdom) || thisdow)) {
 		if ((doNonWild && !(schedule->flags & (MIN_STAR|HR_STAR)))
 		    || (doWild && (schedule->flags & (MIN_STAR|HR_STAR))))
 		{
@@ -1062,6 +1102,22 @@ PollForTasks(List *taskList)
 
 		if (task->state == CRON_TASK_WAITING && task->pendingRunCount == 0)
 		{
+			/*
+			 * Make sure we do not wait past the next run time of an interval
+			 * job.
+			 */
+			if (task->secondsInterval > 0)
+			{
+				TimestampTz nextRunTime =
+					TimestampTzPlusMilliseconds(task->lastStartTime,
+												task->secondsInterval * 1000);
+
+				if (TimestampDifferenceExceeds(nextRunTime, nextEventTime, 0))
+				{
+					nextEventTime = nextRunTime;
+				}
+			}
+
 			/* don't poll idle tasks */
 			continue;
 		}
@@ -1132,9 +1188,11 @@ PollForTasks(List *taskList)
 	pollTimeout = waitSeconds * 1000 + waitMicros / 1000;
 	if (pollTimeout <= 0)
 	{
-		pfree(polledTasks);
-		pfree(pollFDs);
-		return;
+		/*
+		 * Interval jobs might frequently be overdue, inject a small
+		 * 1ms wait to avoid getting into a tight loop.
+		 */
+		pollTimeout = 1;
 	}
 	else if (pollTimeout > MaxWait)
 	{
@@ -1248,6 +1306,8 @@ ManageCronTask(CronTask *task, TimestampTz currentTime)
 			else
 				task->state = CRON_TASK_START;
 
+			task->lastStartTime = currentTime;
+
 			RunningTaskCount++;
 
 			/* Add new entry to audit table. */
@@ -1410,7 +1470,7 @@ ManageCronTask(CronTask *task, TimestampTz currentTime)
 			 * there trying to write the queue long after we've gone away.)
 			 */
 			oldcontext = MemoryContextSwitchTo(TopMemoryContext);
-			shm_mq_attach(mq, task->seg, NULL);
+			task->sharedMemoryQueue = shm_mq_attach(mq, task->seg, NULL);
 			MemoryContextSwitchTo(oldcontext);
 
 			/*
@@ -1660,11 +1720,9 @@ ManageCronTask(CronTask *task, TimestampTz currentTime)
 		case CRON_TASK_BGW_RUNNING:
 		{
 			pid_t pid;
-			shm_mq_handle *responseq;
-			shm_mq *mq;
-			shm_toc *toc;
 
 			Assert(UseBackgroundWorkers);
+
 			/* check if job has been removed */
 			if (jobCanceled(task))
 			{
@@ -1676,28 +1734,28 @@ ManageCronTask(CronTask *task, TimestampTz currentTime)
 				break;
 			}
 
-			toc = shm_toc_attach(PG_CRON_MAGIC, dsm_segment_address(task->seg));
-			#if PG_VERSION_NUM < 100000
-				mq = shm_toc_lookup(toc, PG_CRON_KEY_QUEUE);
-			#else
-				mq = shm_toc_lookup(toc, PG_CRON_KEY_QUEUE, false);
-			#endif
-			responseq = shm_mq_attach(mq, task->seg, NULL);
-
 			/* still waiting for job to complete */
 			if (GetBackgroundWorkerPid(&task->handle, &pid) != BGWH_STOPPED)
 			{
-				GetBgwTaskFeedback(responseq, task, true);
-				shm_mq_detach(responseq);
-				break;
+				bool isRunning = true;
+
+				/* process notices and warnings */
+				ProcessBgwTaskFeedback(task, isRunning);
 			}
+			else
+			{
+				bool isRunning = false;
 
-			GetBgwTaskFeedback(responseq, task, false);
+				/* process remaining notices and final task result */
+				ProcessBgwTaskFeedback(task, isRunning);
 
-			task->state = CRON_TASK_DONE;
-			dsm_detach(task->seg);
-			task->seg = NULL;
-			RunningTaskCount--;
+				task->state = CRON_TASK_DONE;
+
+				dsm_detach(task->seg);
+
+				task->seg = NULL;
+				RunningTaskCount--;
+			}
 
 			break;
 		}
@@ -1866,10 +1924,17 @@ GetTaskFeedback(PGresult *result, CronTask *task)
 	PQclear(result);
 }
 
-static void
-GetBgwTaskFeedback(shm_mq_handle *responseq, CronTask *task, bool running)
-{
 
+/*
+ * ProcessBgwTaskFeedback reads messages from a shared memory queue associated
+ * with the background worker that is executing a given task. If the task is
+ * still running, the function does not block if the queue is empty. Otherwise,
+ * it reads until the end of the queue.
+ */
+static void
+ProcessBgwTaskFeedback(CronTask *task, bool running)
+{
+	shm_mq_handle *responseq = task->sharedMemoryQueue;
 	TimestampTz end_time;
 
 	Size            nbytes;
@@ -1885,11 +1950,15 @@ GetBgwTaskFeedback(shm_mq_handle *responseq, CronTask *task, bool running)
 	 */
 	for (;;)
 	{
+		/* do not wait if the task is running */
+		bool nowait = running;
+
 		/* Get next message. */
-		res = shm_mq_receive(responseq, &nbytes, &data, false);
+		res = shm_mq_receive(responseq, &nbytes, &data, nowait);
 
 		if (res != SHM_MQ_SUCCESS)
 			break;
+
 		initStringInfo(&msg);
 		resetStringInfo(&msg);
 		enlargeStringInfo(&msg, nbytes);
@@ -1978,7 +2047,8 @@ CronBackgroundWorker(Datum main_arg)
 	shm_mq *mq;
 	shm_mq_handle *responseq;
 
-	pqsignal(SIGTERM, pg_cron_background_worker_sigterm);
+	/* handle SIGTERM like regular backend */
+	pqsignal(SIGTERM, die);
 	BackgroundWorkerUnblockSignals();
 
 	/* Set up a memory context and resource owner. */
